@@ -6,17 +6,25 @@ from django.utils import importlib
 
 from datastream import Datastream
 
-from . import signals
+try:
+    from django.db.models.constants import LOOKUP_SEP
+except ImportError:
+    # To support Django 1.4 we move to location where Django 1.5+ has constants
+    import sys
+    from django.db.models.sql import constants
+    import django.db.models
+    django.db.models.constants = constants
+    sys.modules['django.db.models.constants'] = django.db.models.constants
 
 datastream = None
 
-# Load the backend as specified in configuration
-if getattr(settings, 'DATASTREAM_BACKEND', None) is not None:
-    backend = settings.DATASTREAM_BACKEND
+
+def init_datastream(datastream_backend, datastream_backend_settings):
+    backend = datastream_backend
 
     if isinstance(backend, basestring):
         i = backend.rfind('.')
-        module, attr = backend[:i], backend[i+1:]
+        module, attr = backend[:i], backend[i + 1:]
 
         try:
             mod = importlib.import_module(module)
@@ -27,9 +35,10 @@ if getattr(settings, 'DATASTREAM_BACKEND', None) is not None:
         except AttributeError:
             raise exceptions.ImproperlyConfigured('Module "%s" does not define a "%s" datastream backend' % (module, attr))
 
-        backend = cls(**getattr(settings, 'DATASTREAM_BACKEND_SETTINGS', {}))
+        backend = cls(**datastream_backend_settings)
 
-    def callback(stream_id, granularity, datapoint):
-        signals.new_datapoint.send(sender=datastream, stream_id=stream_id, granularity=granularity, datapoint=datapoint)
+    return Datastream(backend)
 
-    datastream = Datastream(backend, callback)
+# Load the backend as specified in configuration
+if getattr(settings, 'DATASTREAM_BACKEND', None) is not None:
+    datastream = init_datastream(settings.DATASTREAM_BACKEND, getattr(settings, 'DATASTREAM_BACKEND_SETTINGS', {}))
