@@ -1,145 +1,105 @@
 from __future__ import absolute_import
 
-import datetime, urllib
+import datetime
 
 import pytz
 
-from tastypie import bundle, exceptions, fields, http, paginator, resources
+from django.conf import settings
+
+from tastypie import bundle as tastypie_bundle, exceptions, fields as tastypie_fields, resources
+
+from . import datastream, fields, paginator, serializers
+from datastream import api as datastream_api, exceptions as datastream_exceptions
 
-from . import datastream, serializers
-from datastream import exceptions as datastream_exceptions
 
 class InvalidGranularity(exceptions.BadRequest):
     pass
 
+
 class InvalidDownsampler(exceptions.BadRequest):
     pass
 
+
 class InvalidRange(exceptions.BadRequest):
     pass
 
-QUERY_GRANULARITY = 'g'
-QUERY_START = 's'
-QUERY_END = 'e'
-QUERY_START_EXCLUSIVE = 'sx'
-QUERY_END_EXCLUSIVE = 'ex'
-QUERY_VALUE_DOWNSAMPLERS = 'v'
-QUERY_TIME_DOWNSAMPLERS = 't'
-QUERY_REVERSE = 'r'
 
-class DatapointsField(fields.ApiField):
-    """
-    A datapoints field.
-    """
+QUERY_GRANULARITY = 'granularity'
+QUERY_START = 'start'
+QUERY_END = 'end'
+QUERY_START_EXCLUSIVE = 'start_exclusive'
+QUERY_END_EXCLUSIVE = 'end_exclusive'
+QUERY_VALUE_DOWNSAMPLERS = 'value_downsamplers'
+QUERY_TIME_DOWNSAMPLERS = 'time_downsamplers'
+QUERY_REVERSE = 'reverse'
 
-    dehydrated_type = 'datapoints'
-    help_text = "A list of datapoints."
 
-class Paginator(paginator.Paginator):
-    def get_slice(self, limit, offset):
-        if limit == 0:
-            # If zero, return nothing
-            return []
+class StreamsList(datastream_api.ResultsBase):
+    def __init__(self, cursor):
+        self.cursor = cursor
 
-        # Optimization
-        self.objects.batch_size(limit)
+    def batch_size(self, batch_size):
+        self.cursor.batch_size(batch_size)
 
-        return list(self.objects[offset:offset + limit])
+    def count(self):
+        return self.cursor.count()
 
-    def _generate_uri(self, limit, offset):
-        if self.resource_uri is None:
-            return None
+    def __iter__(self):
+        for stream in self.cursor:
+            yield datastream.Stream(stream)
 
-        request_params = dict([k, v.encode('utf-8')] for k, v in self.request_data.items())
+    def __getitem__(self, key):
+        if isinstance(key, slice):
+            return StreamsList(self.cursor.__getitem__(key))
+        elif isinstance(key, (int, long)):
+            return datastream.Stream(self.cursor.__getitem__(key))
+        else:
+            raise TypeError
 
-        if 'offset' in request_params:
-            del request_params['offset']
-        if 'limit' in request_params:
-            del request_params['limit']
-
-        request_params.update({'l': limit, 'o': offset})
-
-        return '%s?%s' % (
-            self.resource_uri,
-            urllib.urlencode(request_params)
-        )
-
-    def get_limit(self):
-        if 'l' not in self.request_data:
-            return super(Paginator, self).get_limit()
-
-        limit = self.request_data['l']
-
-        try:
-            limit = int(self.request_data['l'])
-        except ValueError:
-            raise exceptions.BadRequest("Invalid limit '%s' provided. Please provide a positive integer.", limit)
-
-        if limit < 0:
-            raise exceptions.BadRequest("Invalid limit '%s' provided. Please provide an integer >= 0.", limit)
-
-        return limit
-
-    def get_offset(self):
-        if 'o' not in self.request_data:
-            return super(Paginator, self).get_offset()
-
-        offset = self.request_data['o']
-
-        try:
-            offset = int(offset)
-        except ValueError:
-            raise exceptions.BadRequest("Invalid offset '%s' provided. Please provide an integer.", offset)
-
-        if offset < 0:
-            raise exceptions.BadRequest("Invalid offset '%s' provided. Please provide an integer >= 0.", offset)
-
-        return offset
 
 class StreamResource(resources.Resource):
     class Meta:
         list_allowed_methods = ('get',)
-        detail_allowed_methods = ('get', 'wait')
+        detail_allowed_methods = ('get',)
         only_detail_fields = ('datapoints',)
         serializer = serializers.DatastreamSerializer()
-        paginator_class = Paginator
-        limit = 100
+        paginator_class = paginator.Paginator
+        detail_paginator_class = paginator.DetailPaginator
+        detail_limit = getattr(settings, 'API_DETAIL_LIMIT_PER_PAGE', 100)
+        max_detail_limit = 10000
 
     # TODO: Set help text
-    id = fields.CharField(attribute='id', null=False, blank=False, readonly=True, unique=True, help_text=None)
-    value_downsamplers = fields.ListField(attribute='value_downsamplers', null=False, blank=False, readonly=True, help_text=None)
-    time_downsamplers = fields.ListField(attribute='time_downsamplers', null=False, blank=False, readonly=True, help_text=None)
-    highest_granularity = fields.CharField(attribute='highest_granularity', null=False, blank=False, readonly=True, help_text=None)
-    tags = fields.ListField(attribute='tags', null=True, blank=False, readonly=False, help_text=None)
+    id = tastypie_fields.CharField(attribute='id', null=False, blank=False, readonly=True, unique=True, help_text=None)
+    value_downsamplers = tastypie_fields.ListField(attribute='value_downsamplers', null=False, blank=False, readonly=True, help_text=None)
+    time_downsamplers = tastypie_fields.ListField(attribute='time_downsamplers', null=False, blank=False, readonly=True, help_text=None)
+    highest_granularity = tastypie_fields.CharField(attribute='highest_granularity', null=False, blank=False, readonly=True, help_text=None)
+    tags = tastypie_fields.DictField(attribute='tags', null=True, blank=False, readonly=False, help_text=None)
 
-    datapoints = DatapointsField(attribute='datapoints', null=True, blank=False, readonly=True, help_text=None)
+    datapoints = fields.DatapointsField(attribute='datapoints', null=True, blank=False, readonly=True, help_text=None)
 
-    def get_resource_uri(self, bundle_or_obj):
-        kwargs = {
-            'resource_name': self._meta.resource_name,
-        }
+    def detail_uri_kwargs(self, bundle_or_obj):
+        kwargs = {}
 
-        if isinstance(bundle_or_obj, bundle.Bundle):
-            kwargs['pk'] = bundle_or_obj.obj.id
+        if isinstance(bundle_or_obj, tastypie_bundle.Bundle):
+            kwargs[self._meta.detail_uri_name] = bundle_or_obj.obj.id
         else:
-            kwargs['pk'] = bundle_or_obj.id
+            kwargs[self._meta.detail_uri_name] = bundle_or_obj.id
 
-        if self._meta.api_name is not None:
-            kwargs['api_name'] = self._meta.api_name
-
-        return self._build_reverse_url('api_dispatch_detail', kwargs=kwargs)
+        return kwargs
 
     def get_object_list(self, request):
-        # TODO: Provide users a way to query streams by tags
-        return [datastream.Stream(stream) for stream in datastream.find_streams()]
+        # TODO: Provide users a way to query streams by tags (is this the same as allow filtering?) (use ListQuerySet from django-tastypie-mongoengine?)
+        return StreamsList(datastream.find_streams())
 
     def apply_sorting(self, obj_list, options=None):
+        # TODO: Allow sorting (use ListQuerySet from django-tastypie-mongoengine?)
         return obj_list
 
-    def obj_get_list(self, request=None, **kwargs):
-        return self.get_object_list(request)
+    def obj_get_list(self, bundle, **kwargs):
+        return self.get_object_list(bundle.request)
 
     def alter_list_data_to_serialize(self, request, data):
+        data = super(StreamResource, self).alter_list_data_to_serialize(request, data)
         for obj in data['objects']:
             for field_name in self._meta.only_detail_fields:
                 del obj.data[field_name]
@@ -148,10 +108,10 @@ class StreamResource(resources.Resource):
     def alter_detail_data_to_serialize(self, request, data):
         data.data['query_params'] = self._get_query_params(request, data.obj)
 
-        paginator = self._meta.paginator_class(request.GET, data.data['datapoints'], resource_uri=data.data['resource_uri'], limit=self._meta.limit)
+        paginator = self._meta.detail_paginator_class(request.GET, data.data['datapoints'], resource_uri=data.data['resource_uri'], limit=self._meta.detail_limit, max_limit=self._meta.max_detail_limit, collection_name='datapoints')
         page = paginator.page()
 
-        data.data['datapoints'] = page['objects']
+        data.data['datapoints'] = list(page['datapoints'])
         data.data.setdefault('meta', {}).update(page['meta'])
 
         return data
@@ -159,6 +119,9 @@ class StreamResource(resources.Resource):
     def _get_query_params(self, request, stream):
         granularity = request.GET.get(QUERY_GRANULARITY, None)
         for g in datastream.Granularity.values:
+            if granularity == g.name:
+                granularity = g
+                break
             if granularity == g.key:
                 granularity = g
                 break
@@ -194,12 +157,18 @@ class StreamResource(resources.Resource):
         if end and end_exclusive:
             raise InvalidRange("Only one time range end can be specified.")
 
+        if not start and not start_exclusive:
+            start = datetime.datetime.min
+
         reverse = request.GET.get(QUERY_REVERSE, '0').lower() in ('yes', 'true', 't', '1', 'y')
 
         value_downsamplers = []
         for downsampler in request.GET.getlist(QUERY_VALUE_DOWNSAMPLERS, []):
             for d in downsampler.split(','):
                 for name, key in datastream.VALUE_DOWNSAMPLERS.items():
+                    if d == name:
+                        value_downsamplers.append(name)
+                        break
                     if d == key:
                         value_downsamplers.append(name)
                         break
@@ -213,6 +182,9 @@ class StreamResource(resources.Resource):
         for downsampler in request.GET.getlist(QUERY_TIME_DOWNSAMPLERS, []):
             for d in downsampler.split(','):
                 for name, key in datastream.TIME_DOWNSAMPLERS.items():
+                    if d == name:
+                        time_downsamplers.append(name)
+                        break
                     if d == key:
                         time_downsamplers.append(name)
                         break
@@ -233,19 +205,16 @@ class StreamResource(resources.Resource):
             'time_downsamplers': time_downsamplers,
         }
 
-    def obj_get(self, request=None, **kwargs):
+    def obj_get(self, bundle, **kwargs):
         try:
             stream = datastream.Stream(datastream.get_tags(kwargs['pk']))
         except datastream_exceptions.StreamNotFound:
             raise exceptions.NotFound("Stream '%s' not found." % kwargs['pk'])
 
-        params = self._get_query_params(request, stream)
-
-        if not params['start'] and not params['start_exclusive']:
-            params['start'] = datetime.datetime.min
+        params = self._get_query_params(bundle.request, stream)
 
         stream.datapoints = datastream.get_data(
-            stream_id=kwargs['pk'],
+            stream_id=stream.id,
             granularity=params['granularity'],
             start=params['start'],
             end=params['end'],
@@ -258,21 +227,17 @@ class StreamResource(resources.Resource):
 
         return stream
 
-    def obj_create(self, bundle, request=None, **kwargs):
+    def obj_create(self, bundle, **kwargs):
         raise NotImplementedError
 
-    def obj_update(self, bundle, request=None, **kwargs):
+    def obj_update(self, bundle, **kwargs):
         raise NotImplementedError
 
-    def obj_delete_list(self, request=None, **kwargs):
+    def obj_delete_list(self, bundle, **kwargs):
         raise NotImplementedError
 
-    def obj_delete(self, request=None, **kwargs):
+    def obj_delete(self, bundle, **kwargs):
         raise NotImplementedError
 
     def rollback(self, bundles):
         raise NotImplementedError
-
-    def wait_detail(self, request, **kwargs):
-        # TODO: Implement
-        return http.HttpNotImplemented()
